;; -*- geiser-scheme-implementation: guile -*-

;; level.scm - This file is part of Guile TD
;; Copyright (C) 2017  Christopher Howard

;; Guile TD is free software: you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; Guile TD is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

(define-module (td level)
  #:use-module (td hooks)
  #:use-module (td enemy)
  #:use-module (td tower)
  #:use-module (td gfx)
  #:use-module (td picture)
  #:use-module (aa-tree)
  #:use-module (chickadee)
  #:use-module (chickadee math vector)
  #:use-module (chickadee scripting)
  #:use-module (chickadee scripting script)
  #:use-module (chickadee render sprite)
  #:use-module (chickadee render texture)
  #:use-module (chickadee render font)
  #:use-module (oop goops)
  #:export (failed-game?
            fails-text
            game-complete))

(define max-to-cross 10)

(define total-crossed 0)
(define current-level-script #nil)
(define game-complete #f)

(define path1 '((992 544) (800 544) (800 416) (864 416) (864 160)
                (672 160) (672 224) (480 224) (480 352) (544 352)
                (544 544) (160 544) (160 416) (288 416) (288 160)
                (160 160) (160 224) (32 224)))

(define path1-river-pre '((4 0) (6 0) (6 6) (9 6) (9 8) (11 8) (11 11)))

(define path1-river '((288 32) (416 32) (416 416) (608 416) (608 544) (736 544) (736 736)))

(define path1-center '((992 352) (32 352)))
(define path1-upper '((992 224) (32 224)))
(define path1-lower '((992 480) (32 480)))

(add-hook! load-hook (lambda () (run-hook new-game-hook)))

(define (failed-game?)
  (>= total-crossed max-to-cross))

(add-hook! field-crossed
           (lambda ()
             (set! total-crossed (1+ total-crossed))
             (if (failed-game?)
                 (run-hook game-fail))))

(add-hook! new-game-hook
           (lambda ()
             (set! total-crossed 0)))

(define (fails-text)
  (make <text>
    #:font font-gns
    #:string (string-append "Fails: "
                            (number->string total-crossed)
                            " of "
                            (number->string max-to-cross))))

(define (blocked-tiles-to-tower-selection-grid cols blk-lst)
  (letrec ((f (lambda (bl col row tg)
                (let ((end-of-row (= (1+ col) cols)))
                  (if (nil? bl)
                      tg
                      (f (cdr bl)
                         (if end-of-row
                             0
                             (1+ col))
                         (if end-of-row
                             (1+ row)
                             row)
                         (if (nil? (car bl))
                             tg
                             (aa-insert tg
                                        (list col row)
                                        (car bl)))))))))
    (f blk-lst 0 0 #nil)))

(define level1-blocked-tiles
  '(#nil #nil #nil #nil :blk :blk :blk #nil #nil #nil #nil #nil #nil #nil #nil #nil
         #nil #nil #nil #nil #nil #nil :blk #nil #nil #nil #nil #nil #nil #nil #nil #nil
         #nil #nil :blk :blk :blk #nil :blk #nil #nil #nil :blk :blk :blk :blk #nil #nil
         :blk :blk :blk #nil :blk #nil :blk :blk :blk :blk :blk #nil #nil :blk #nil #nil
         #nil #nil #nil #nil :blk #nil :blk :blk #nil #nil #nil #nil #nil :blk #nil #nil
         #nil #nil #nil #nil :blk #nil :blk :blk :blk #nil #nil #nil #nil :blk #nil #nil
         #nil #nil :blk :blk :blk #nil :blk :blk :blk :blk #nil #nil :blk :blk #nil #nil
         #nil #nil :blk #nil #nil #nil #nil #nil :blk :blk #nil #nil :blk #nil #nil #nil
         #nil #nil :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk
         #nil #nil #nil #nil #nil #nil #nil #nil #nil #nil #nil :blk #nil #nil #nil #nil
         :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk
         :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk :blk))

(define level1-tower-selection-grid (blocked-tiles-to-tower-selection-grid 16 level1-blocked-tiles))

(add-hook! new-game-hook
           (lambda ()
             (set! tower-selection-grid level1-tower-selection-grid)))

(define (min2ms min)
  (* min 60000))

(define (sec2ms sec)
  (* sec 1000))

(define (spaced-repeat iter ms proc)
  (letrec ((fn
            (lambda (i)
              (unless (< i 1)
                (proc)
                (sleep ms)
                (fn (1- i))))))
    (fn iter)))

(define (sleep-until-no-enemies)
  (while (not (nil? enemies))
    (sleep 1000)))

(define (sleep20sec) (sleep 20000))

(define level1-script
  (lambda ()
    (set! current-level-script
      (script
       (sleep (sec2ms 5))

       ;; w1; 3s; 18p
       (spaced-repeat 3 (sec2ms 2)
                      (lambda () (launch-soldier! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w2; 36p
       (spaced-repeat 6 (sec2ms 2)
                      (lambda () (launch-soldier! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w3; 60p
       (spaced-repeat 10 (sec2ms 2)
                      (lambda () (launch-soldier! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w4; 120p
       (spaced-repeat 3 1500
                      (lambda () (launch-tracked-soldier! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w5; 160p
       (spaced-repeat 4 800
                      (lambda () (launch-trike! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w6; 240p
       (spaced-repeat 8 (sec2ms 2)
                      (lambda () (launch-soldier-l2! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w7; 360p
       (spaced-repeat 6 1500
                      (lambda () (launch-tank! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w8; 480p
       (spaced-repeat 12 800
                      (lambda ()
                        (launch-trike! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w9; 660p
       (spaced-repeat 22 (sec2ms 2)
                      (lambda ()
                        (launch-soldier-l2! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w10; 840p
       (spaced-repeat 14 1500
                      (lambda () (launch-tank! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w11; 1000p
       (spaced-repeat 11 800
                      (lambda ()
                        (launch-ptboat! path1-river)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w12; 1400p
       (spaced-repeat 7 500
                      (lambda () (launch-helo! path1-center)))
       (sleep-until-no-enemies)
       (sleep20sec)       
       
       ;; w13; 1680p
       (spaced-repeat 14 1500
                      (lambda () (launch-tank! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w14; 1980p
       (spaced-repeat 33 (sec2ms 2)
                      (lambda () (launch-soldier-l3! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w15; 2800p
       (spaced-repeat 28 800
                      (lambda ()
                        (launch-ptboat! path1-river)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w16; 2880
       (spaced-repeat 16 1500
                      (lambda () (launch-tank-l3! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w17; 3400p
       (spaced-repeat 17 500
                      (lambda () (launch-helo! path1-center)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w18; 3900p
       (spaced-repeat 13 3000
                      (lambda () (launch-destroyer! path1-river)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w19; 4500p
       (spaced-repeat 25 1500
                      (lambda () (launch-tank-l3! path1)))
       (sleep-until-no-enemies)
       (sleep20sec)

       ;; w20; 5220p
       (script 
        (spaced-repeat 14 1500
                       (lambda () (launch-tank-l3! path1))))
       (script 
        (spaced-repeat 9 3000
                       (lambda () (launch-destroyer! path1-river))))
       (sleep20sec)
       (sleep-until-no-enemies)

       (sleep (sec2ms 3))
       (set! game-complete #t)
       (run-hook game-victory-hook)
))))

(add-hook! new-game-hook
           (lambda ()
             (set! game-complete #f)
             (unless (nil? current-level-script)
               (cancel-script current-level-script))
             (level1-script)))
